<?php
/*
See yak-for-wordpress.php for information and license terms
*/

/**
 * A singleton registry to store app-wide data 
 */
class Registry {
    var $_objects = array();

    function &getInstance() {
        static $self;

        if (is_object($self) == true) {
            return $self;
        }

        $self = new Registry;
        return $self;
    }
    
    function set($name, &$object) {
        $this->_objects[$name] =& $object;
    }

    function &get($name) {
        return $this->_objects[$name];
    }
}


/**
 * Return a default if the specified value is not set
 *
 * @param $val the value to return a default if empty
 * @param $def the default value to use if $val is not set
 */
function yak_default($val, $def = '') {
    if (empty($val)) {
        return $def;   
    }
    else {
        return $val;   
    }
}


if (!function_exists('yak_escape_string')) {
    /**
     * Properly escape a string so it can be used in a SQL statement 
     */
    function yak_escape_string($string) {
        if (version_compare(phpversion(),"4.3.0") == "-1") {
            return mysql_escape_string($string);
        }
        else {
            return mysql_real_escape_string($string);
        }
    }
}


if (!function_exists('yak_fix_escaping')) {
    /**
     * Fix escaping of single-quotes
     */
    function yak_fix_escaping($s) {
        $s = str_replace("\\'", "'", $s);
        $s = str_replace("\\\\", "\\", $s);
        return $s;
    }
}

if (!function_exists('yak_sql_escape')) {
    function yak_sql_escape($s) {
        $s = str_replace("\'", "''", $s);
        return $s;
    }
}


if (!function_exists('yak_get_country')) {
    /**
     * Return the name of a country by the country code
     */
    function yak_get_country($country_code) {
        global $countries;
        return $countries[$country_code];
    }
}


if (!function_exists('yak_get_country_by_name')) {
    /**
     * Return a country code by the country name (if not found, return null)
     */
    function yak_get_country_by_name($ctyname) {
        global $countries;
        
        foreach ($countries as $code=>$name) {
            if ($name == $ctyname) {
                return $code;   
            }
        }
        
        return null;
    }
}


if (!function_exists('yak_overlap')) {
    /**
     * Append 2 strings, removing any overlap between the two
     */
    function yak_overlap($s1, $s2) {
        for ($x = strlen($s1)-1; $x >= 0; $x--) {
            $end = substr($s1, $x);
            $start = substr($s2, 0, strlen($end));
            if ($start == $end) {
                return substr($s1, 0, $x) . $s2;
            }
        }
        return $s1 . $s2;
    }
}

/**
 * Convert a memory value as a number of bytes.
 */
function yak_return_bytes($val) {
    $val = trim($val);
    $last = strtolower($val{strlen($val)-1});
    switch($last) {
        // The 'G' modifier is available since PHP 5.1.0
        case 'g':
            $val *= 1024;
        case 'm':
            $val *= 1024;
        case 'k':
            $val *= 1024;
    }

    return $val;
}


if (!function_exists('yak_str_contains')) {
    /**
     * Return true if one string can be found in another
     *
     * @param $haystack the string to search *in*
     * @param $needle the string to search *for*
     */
    function yak_str_contains($haystack, $needle){
        $pos = strpos($haystack, $needle);
        
        if ($pos === false) {
            return false;
        }
        else {
            return true;
        }
    }   
}


if (!function_exists('yak_do_http')) {
    /**
     * Execute an HTTP method.  If curl is available, use curl, otherwise pfsockopen.
     *
     * @param $host the server name/ip
     * @param $uri the uri to invoke
     * @param $params the parameters to use in the execution
     * @param $headers the HTTP headers
     * @param $method the HTTP method
     */
    if (function_exists('curl_init')) {
        function yak_do_http($host, $uri, $params, $headers = null, $method = 'POST') {
            $ch = curl_init();
              
            if ($method == 'POST') {
                curl_setopt($ch, CURLOPT_URL, $host . $uri);
                curl_setopt($ch, CURLOPT_POST, 1);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
            }
            else {
                curl_setopt($ch, CURLOPT_URL, $host . $uri . '?' . $params);
                curl_setopt($ch, CURLOPT_GET, 1);
            }
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1); 
            
            $proxy_url = yak_get_option(HTTP_PROXY_URL, '');
            if ($proxy_url != '') {
                curl_setopt($ch, CURLOPT_PROXYTYPE, CURLPROXY_HTTP);
                curl_setopt($ch, CURLOPT_PROXY, $proxy_url);    
            }
            
            $response = curl_exec($ch);
            
            if (curl_errno($ch)) {
                return curl_error($ch);
            }
            else {
                return $response;
            }
        }
    }
    else {
        function yak_do_http($host, $uri, $params, $headers = null, $method = 'POST') {
            $parsed = parse_url($host);
            $hostname = $parsed['host'];
            $port = 80;
            if ($parsed['scheme'] == 'https') {
                $host = 'ssl://' . $hostname;
                $port = 443;
            }
            
            $header = "Host: $hostname\r\n";
            $header .= "Accept-Encoding: identity\r\n";
            
            # compose HTTP request header
            if ($method == 'POST') {
                $header .= "User-Agent: PHP Script\r\n";
                $header .= "Content-Type: application/x-www-form-urlencoded\r\n";
                $header .= "Content-Length: " . strlen($params) . "\r\n";
            }
            else {
                $uri .= '?' . $params;
                $params = '';
            }
            
            if ($headers != null) {
                foreach ($headers as $key=>$val) {
                    $header .= $key . ": " . $val . "\r\n";       
                }
            }
            
            $header .= "Connection: close\r\n\r\n";

            $fp = pfsockopen($host, $port, $errno, $errstr);
            if (!$fp) {
                error_log("comms error $errstr ($errno)");
                return "ERROR: $errstr ($errno)";
            }
            else { 
                fputs($fp, "$method $uri HTTP/1.1\r\n");
                fputs($fp, $header . $params);
                fwrite($fp, $out);
                
                $response = '';
                while (!feof($fp)) {
                    $response .= fgets($fp, 128);
                }
                fclose($fp);
                
                $response = split("\r\n\r\n", $response);
                $header = $response[0];
                $responsecontent = $response[1];
                if (!(strpos($header, "Transfer-Encoding: chunked") === false)) {
                    $aux = split("\r\n", $responsecontent);
                    for ($i = 0; $i < count($aux); $i++) {
                        if ($i == 0 || ($i % 2 == 0)) {
                            $aux[$i] = "";
                        }
                    }
                    $responsecontent = implode("", $aux);
                }
                
                $rtn = chop($responsecontent);
                
                error_log($rtn);
                
                return $rtn;
            }   
        }
    }
}


if (!function_exists('yak_get_tag_value')) {
    function yak_get_tag_value($tag_start, $tag_end, $content, $start_from = 0) {
        $tag_start_len = strlen($tag_start);
        $pos = strpos($content, $tag_start, $start_from);
        if ($pos >= 0) {
            $pos2 = strpos($content, $tag_end, $pos + $tag_start_len);
            return substr($content, $pos + $tag_start_len, $pos2 - ($pos + $tag_start_len));
        }
        else {
            return null;
        }
    }
}


if (!function_exists('yak_get_url')) {
    /**
     * Process a url to make sure we handle it intelligently.
     *     1. if there is no protocol, then assume this is a url for the local site.
     *     2. if we're going to append to the url, add either ? or & to the end depending upon
     *        what's already in the url
     *
     * @param $url the initial url to use
     * @param $safe_append are we going to append parameters to the url?
     */
    function yak_get_url($url, $safe_append = false) {
        if (strpos($url, 'http://') === false && strpos($url, 'https://') === false) {
            $rtn = get_bloginfo('wpurl') . $url;
        }
        else {
            $rtn = $url;
        }
        
        if ($safe_append == true) {
            if (strpos($rtn, '?') === false) {
                $rtn = $rtn . '?';
            }
            else {
                $rtn = $rtn . '&';
            }
        }
        
        return $rtn;
    }
}


if (!function_exists('yak_encode_params')) {
    /**
     * Encode a param array as a url-encoded string
     */
    function yak_encode_params($param_array) {
        $params = "";
        foreach( $param_array as $key => $value ) {
            $params .= "$key=" . urlencode( $value ) . "&";
        }
        return $params;
    }
}

if (!function_exists('yak_decode_params')) {
    /**
     * Decode a param string into an array -- note: ignores duplicates (http params can have
     * dups, so this isn't correct behaviour, but works for YAK's purposes... for the moment)
     */
    function yak_decode_params($params) {
        $rtn = array();
        foreach (explode('&', $params) as $keyval) {
            $kv = explode('=', $keyval);
            $rtn[$kv[0]] = urldecode($kv[1]);
        }
        return $rtn;
    }
}


if (!function_exists('yak_date_control')) {
    function yak_date_control($name, $year, $month, $day, $include_day = true) {
        global $months;

    	echo yak_html_select(null, 'month_' . $name, $month, $months);
    	if ($include_day) {
    	    echo '<input type="text" name="day_' . $name . '" value="' . $day . '" maxlength="2" size="4" />, ';
	    }
    	echo '<input type="text" name="year_' . $name . '" value="' . $year . '" maxlength="4" size="6" />';
    }
}


if (!function_exists('yak_address_hidden_input')) {
    function yak_address_hidden_input($address, $type) {
        if ($address != null) {
            foreach ($address->get_members() as $key=>$value) {
                echo '<input type="hidden" name="' . $type . '_' . $key . '" value="' . $value . '" />';
            }
        }
    }
}

if (!function_exists('yak_in_list')) {
    function yak_in_list($name, $typeList) {
        foreach ($typeList as $type) {
            if ($name == $type->name) {
                return true;
            }
        }
        return false;
    }
}


if (!function_exists("yak_sendmail")) {
    function yak_sendmail($from, $replyto, $to, $subject, $message) {
        if (yak_str_contains($message, '<html')) {
            yak_sendhtml($from, $replyto, $to, $subject, $message);
        }
        else {
            $headers = "MIME-Version: 1.0\r\n" .
                       "From: $from\r\n" .
                       "Reply-To: $replyto\r\n" .
                       "Content-Type: text/plain; charset=UTF-8\r\n" .
                       "Content-Transfer-Encoding: 8bit\r\n";

            wp_mail($to, $subject, $message, $headers);
        }
    }
}

if (!function_exists("yak_sendhtml")) {
    function yak_sendhtml($from, $replyto, $to, $subject, $message) {
        $boundary = uniqid("HTMLEMAIL");
        
        $headers = "MIME-Version: 1.0\r\n"
                 . "From: $from\r\n"
                 . "Reply-To: $replyto\r\n"; 

        $headers .= "Content-Type: multipart/alternative;"
                 .  "boundary = $boundary\r\n\r\n"; 

        $headers .= "This is a MIME encoded message.\r\n\r\n"; 

        // plain text version first
        $headers .= "--$boundary\r\n".
                    "Content-Type: text/plain; charset=UTF-8\r\n".
                    "Content-Transfer-Encoding: base64\r\n\r\n"; 

        $headers .= chunk_split(base64_encode(strip_tags($message))); 

        // html version
        $headers .= "--$boundary\r\n".
                    "Content-Type: text/html; charset=UTF-8\r\n".
                    "Content-Transfer-Encoding: base64\r\n\r\n"; 

        $headers .= chunk_split(base64_encode($message));
        
        wp_mail($to, $subject, $message, $headers);
    }
}
?>